;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#xdd30db0ff0717a4a50d0bd07723ba4a1 . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#x7a1cbad950d579af860557602c3e2f16 . "./code.cg2js/cg-gesture-emulation.fasl")
(#x04b663a2c14f76100611bf032ad1783b . "./code.cg2js/cg-grid-widget.fasl")
(#x6138de7c4bcf852947819c04ce8689e9 . "./code.cg2js/cg-utility-dialog.fasl")
(#x9699304f4b5e49fc9bc71f1745052043 . "./code.cg2js/cg-wrap-string.fasl")
(#x1850e3d68d7023db71d27ef0cb476466 . "./code.cg2js/cg-editable-text.fasl")
(#x086cbcccb0e8fbf91a5029482e31f2b3 . "./code.cg2js/cg-base.fasl")
(#xb6e610ebf8249c183f7661caeadd5647 . "./code.cg2js/cg-lisp-text.fasl")
(#x69d12270868fbf922b849955faf8b53e . "./code.cg2js/cg-intersect.fasl")
(#x0869deaa1d5b83903c75c1a65cf37f10 . "./code.cg2js/cg-choose-printer.fasl")
(#xbc1b1dff4d57f3079c3363d1cb6acc02 . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#xb09d8036116b27d111fce7cdf4bdca5d . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#xdd7a48bd6aac80590d819018055c2ffb . "./code.cg2js/cg-font-dialog.fasl")
(#x98d0bdd42b10ae2826969c92094b4574 . "./code.cg2js/cg-comtab.fasl")
(#x91f883a73164a239fa2e5724eb521d8c . "./code.cg2js/cg-tooltip.fasl")
(#xfd8b91fc35c81e333bf2797eb8b2fd4c . "./code.cg2js/cg-chart-or-plot.fasl")
(#x57e67ffff72eb2b204fa149bf5c3d100 . "./code.cg2js/cg-get-pixmap.fasl")
(#xf6484728cff348e748e3a2c836ea4049 . "./code.cg2js/cg-status-bar.fasl")
(#xb38f0d05b474e4d0a26d5c656616ed17 . "./code.cg2js/cg-tab-control.fasl")
(#x2a07981899009a61b3808ce2b1a0f944 . "./code.cg2js/cg-icon-pixmap.fasl")
(#x9e65166ccbccc6ee6b9a2cecbbed2884 . "./code.cg2js/cg-list-view.fasl")
(#xa46be3ff110787a4ba34333376238545 . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#x32fc4ee65182f05be4756c24406e4375 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#x224f211fedeeef65eae61efd3099ed0a . "./code.cg2js/cg-project-window.fasl")
(#x157abefae648d199711c8d73d0600d9a . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#xa711fe62120efaaa8e399b6bd21b0026 . "./code.cg2js/cg-drag-and-drop.fasl")
(#x00b0ae22a47f37fdd05fc6a59b734b90 . "./code.cg2js/cg-scaling-stream.fasl")
(#x46c77548708a201e6d24d390b8a334fa . "./code.cg2js/cg-bitmap-stream.fasl")
(#xa825055b1f7139a2ca08a77e2db742e7 . "./code.cg2js/cg-menu.fasl")
(#x2f91b806fa52193960b4cf2763969700 . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#x193aaf1bae05851bc9ad4202f842c1fd . "./code.cg2js/cg-intersect-posbox.fasl")
(#x207290023db5ca52e0cd1a1f7c0a0c2a . "./code.cg2js/cg-split-bar.fasl")
(#x09cce7c3f8c1c24ec32d8dd2dc29b4cd . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#x683204e20887001fd7e6f0e08db743a1 . "./code.cg2js/cg-timer.fasl")
(#xe2c0751378990a8c62a68ba69c8260e0 . "./code.cg2js/cg-text-or-combo.fasl")
(#x115407d722fdb326977158526f4ef669 . "./code.cg2js/cg-picture-widget-palette.fasl")
(#x5835e201de9117edb526524ad06537a1 . "./code.cg2js/cg-string-dialog.fasl")
(#x6b04ef52c8a7d4ca62053724c6a22d49 . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#x03c82b8aeb97e9fa8c94a2968a09d377 . "./code.cg2js/cg-menu-tooltip.fasl")
(#x214cb63e63742dd10dbbce6dc37cdf49 . "./code.cg2js/cg-outline.fasl")
(#x9d0017a1cbf9b858ac2d5ef91a51061b . "./code.cg2js/cg-lettered-menu.fasl")
(#xa95638a831d2ac56ce47b7897e06cc1b . "./code.cg2js/cg-graphics-context.fasl")
(#x9f6209d9d5cc418469e07327ed3db272 . "./code.cg2js/cg-item-list.fasl")
(#x4cdff239ee819cea0821c64474f49110 . "./code.cg2js/cg-caret.fasl")
(#x34072b9bc25052a9e8b0370f9005e52a . "./code.cg2js/cg-printing.fasl")
(#x68a0048ee17442c7d563dc08fdef98d9 . "./code.cg2js/cg-pixmap-printing.fasl")
(#x0574062c467f399d895fe383eb09337c . "./code.cg2js/cg-progress-indicator.fasl")
(#x9bb0d4d7daba830f625950dcf25fbc7c . "./code.cg2js/cg-dde-utils.fasl")
(#x196e485dd968609b7e44061051b91f00 . "./code.cg2js/cg-plot-widget.fasl")
(#x162bb080819536d5525344febf677ddf . "./code.cg2js/cg-os-window.fasl")
(#x800bf6d855cea754485b7b4b5fd40e11 . "./code.cg2js/cg-calendar.fasl")
(#x446af4847b11969012f216c57866379d . "./code.cg2js/cg-chart-widget.fasl")
(#x80c4b84577d2936ad537bf130a9c4078 . "./code.cg2js/cg-file-dialog.fasl")
(#xefef8d22e7e6a75b507ad86ce6657929 . "./code.cg2js/cg-nodes.fasl")
(#x2baedb87f5f983d73e009d68d4755c84 . "./code.cg2js/cg-lamp.fasl")
(#xa708be9026495cbd1ebed712e106015d . "./code.cg2js/cg-html-dialog.fasl")
(#x77395d0cd0d6f8255884ecc868716647 . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#xadd45a1f1c148921ebd8a9cbdd9e57dd . "./code.cg2js/cg-class-slot-grid.fasl")
(#x09b4d9f07e691521e9daa9c240b8ffd6 . "./code.cg2js/cg-choice-list.fasl")
(#xf0f8b05bbc48b066bed1588ab5578166 . "./code.cg2js/cg-multi-picture-button.fasl")
(#x8c51aa1059bd6ac47232f1dbe89ceb64 . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#x4dee4ed7a2462223ed04a222cec1daa7 . "./code.cg2js/cg-hotspot.fasl")
(#x3f1e37d3641897668348fd65f71f8edd . "./code.cg2js/cg-property.fasl")
(#x9b69f87c993ff34ab66e664b3e3ea907 . "./code.cg2js/cg-dropping-outline.fasl")
(#x0aac84d7d3fa1e4b3583b77a20bfe79f . "./code.cg2js/cg-template-string.fasl")
(#xe02d1074b943309e303475a3745c2dd7 . "./code.cg2js/cg-palette.fasl")
(#xb904b3637bfc9ad879777ca2760090e4 . "./code.cg2js/cg-selected-object.fasl")
(#xce6771562a1bdb23a6567a8e368d0f9c . "./code.cg2js/cg-up-down-control.fasl")
(#x723dbcba71a959bc122dae1b25326fc3 . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#xd0d69a6fc7f1dd51fb10689320f68018 . "./code.cg2js/cg-layout.fasl")
(#x2f0a28fdbc57a497a392bda4f4f695d1 . "./code.cg2js/cg-get-position.fasl")
(#x1afa699497425a5a526b0b0ccbc9b32c . "./code.cg2js/cg-lisp-widget.fasl")
(#x945658b68e204e23b38939947fd64b50 . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#xed1c2b67ffe95f2103f95879811ef86d . "./code.cg2js/cg-object-editor.fasl")
(#x015a1bd6d99dd0c8bf0e8bdc7a2f4b18 . "./code.cg2js/cg-class-grid.fasl")
(#xc3cfc285ab5c432dc9aa32b2b9555bcf . "./code.cg2js/cg-yes-no-string.fasl")
(#x3aff77d5ceb24544b3778c3f35ac4dda . "./code.cg2js/cg-toggling-widget.fasl")
(#xa07b640c760987b24d544d1f64d4309c . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#xcef349f3515364a19dd590a680918dc4 . "./code.cg2js/cg-button.fasl")
(#x44c8c7e6e72d4ae28482cc19fa29a5a9 . "./code.cg2js/cg-object-editor-layout.fasl")
(#x649c0a6eaa07c9bf0d1c22941d0f78bf . "./code.cg2js/cg-scroll-bar.fasl")
(#xcc9bb100fa155ffc7acff2bea93249af . "./code.cg2js/cg-combo-box.fasl")
(#x04f03c91cc586237a52b1de1b2335767 . "./code.cg2js/cg-web-browser.fasl")
(#x1165782888c2812953e1560db0a63907 . "./code.cg2js/cg-drawable.fasl")
(#xe382fd5090d812616719f77f9f5cad54 . "./code.cg2js/cg-cursor-pixmap.fasl")
(#xd9d99e4b240bcd8c6df731cfca7cb962 . "./code.cg2js/cg-message-dialog.fasl")
(#xaaf9b789913371530062ce234da2096e . "./code.cg2js/cg-text-widget.fasl")
(#xa723657cdcfc78d605ee97f1523b2d1d . "./code.cg2js/cg-static-text.fasl")
(#x72de722457d2c7d0eb9bf54240ad4bdc . "./code.cg2js/cg-class-support.fasl")
(#x289cc81d38db77ee6333598a9051b646 . "./code.cg2js/cg.fasl")
(#x314d79ac9d05d29d1914596a913fd081 . "./code.cg2js/cg-directory-dialog.fasl")
(#xa2018d03867baa399aa42faa0b43c939 . "./code.cg2js/cg-clipboard.fasl")
(#x2a5556033afb2fdef0517e89040a6ace . "./code.cg2js/cg-drawable-clipboard.fasl")
(#xa2d44622fa7f5ed0cb3f4b7fae26dc30 . "./code.cg2js/cg-bitmap-pane.fasl")
(#x5e1ee1f57f37efe6827651634ba2378a . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#x6f8bffbcffa904956615cbc150833f16 . "./code.cg2js/cg-check-box.fasl")
(#xae7cfe31b7f0e667ec366a82dff6865d . "./code.cg2js/cg-color-dialog.fasl")
(#x5106c65e072622ef66570e9afab304aa . "./code.cg2js/cg-text-edit-pane.fasl")
(#xe0fc0f1c3dbb4940be564966a328d1d7 . "./code.cg2js/cg-dialog-item.fasl")
(#x9fc2079aa3a1e8c0969e3fad0b45a64e . "./code.cg2js/cg-edit-in-place.fasl")
(#x0529eb91e1c2c56ba444d563ccea04a0 . "./code.cg2js/cg-tray.fasl")
(#xc167530c611458843c7ccbb36aadfa46 . "./code.cg2js/cg-paren-matching.fasl")
(#x44b64071a0bd7c4da7ba78b639784f9c . "./code.cg2js/cg-player.fasl")
(#x54f5fb397814cec25b90e2be6aaa1a60 . "./code.cg2js/cg-yes-no-list.fasl")
(#x2164d23987381005500b8dcd4055293c . "./code.cg2js/cg-common-control.fasl")
(#x84f88d4126edbed9394105ad7fd6150b . "./code.cg2js/cg-pixmap-file-io.fasl")
(#xad0c6ea969d2128a3d9a273e4af8b58a . "./code.cg2js/cg-os-widget.fasl")
(#xe13fcb7bd0ec91d818475477327b8372 . "./code.cg2js/cg-sample-file-menu.fasl")
(#x92b174041f934bad55c0a723b70ffc13 . "./code.cg2js/cg-trackbar.fasl")
(#x225812f2e5d2cedf607ae5652a2e8f5f . "./code.cg2js/cg-find-string-dialog.fasl")
(#x5acd62be59be0ec4dc5cc30b4358befe . "./code.cg2js/cg-html-widget.fasl")
(#x26c37b76cd8edde0f8a5eba5e79e1fdb . "./code.cg2js/cg-directory-dialog-os.fasl")
(#x912a645d778931294d269ace1a2cda95 . "./code.cg2js/cg-acache.fasl")
(#x08dd0d9f0ed7dc75fa8e83687b73861c . "./code.cg2js/cg-shortcut-menu.fasl")
(#x5af29a148a350cd331f8908131339142 . "./code.cg2js/cg-pixmap-widget.fasl")
(#x93f85b3330d6e829d8f790a0de5f26ba . "./code.cg2js/cg-icon.fasl")
(#xdf84a17a93eb799f670095563bc92ab7 . "./code.cg2js/cg-pixmap.fasl")
(#x178054053b28d6513ae720758f379aba . "./code.cg2js/cg-clipboard-stack.fasl")
(#xb7dbb62e16785463aad8717ac20fb08e . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#x871daed7eb85cebe820b51ab007f79e8 . "./code.cg2js/cg-toolbar.fasl")
(#x8c23a2d05071623ed95ea672c1eab81f . "./code.cg2js/cg-radio-button.fasl")
(#x097996248295d7e0868579578a01414b . "./code.cg2js/cg-curve.fasl")
(#x06dc9fdfc43508fe54d9c6458d94b077 . "./code.cg2js/cg-picture-widget.fasl")
(#x3ee3d4703835779c2a8ca3eeeb30221d . "./code.cg2js/cg-dev.fasl")
(#xde63eed7fc3685c0b95e3b5b0929bc3a . "./code.cg2js/cg-group-box.fasl")
(#x286ea46ccbf27b632e885b2413063bc2 . "./code.cg2js/cg-fill-texture.fasl")
(#x61943cf2767daf85bace324394b53b9f . "./code.cg2js/cg-pixmap-rotate.fasl")
(#x1b4e291dfe19f6981f9d73ffc730ce90 . "./code.idejs/ide-keyshort.fasl")
(#x0e505ee67d06d32dc0d750cd02345e3c . "./code.idejs/ide-form.fasl")
(#x31ec27034ca10305fcc92df4ea4f04a0 . "./code.idejs/ide-find-in-files.fasl")
(#xd1385ad2588baaceb52fbf78029bc385 . "./code.idejs/ide-class-grapher.fasl")
(#xe94c220cb4190d60be0eae298c72e354 . "./code.idejs/ide-base.fasl")
(#x073c3e425643df4a2756f39d0d9c2b55 . "./code.idejs/ide-class-browser.fasl")
(#xf436a63ec67288aa38393fff5fcd06de . "./code.idejs/ide-find-definitions.fasl")
(#x5c3508d6e6267c9a3395fd798ecdaca3 . "./code.idejs/ide-package-list.fasl")
(#xc51c993a7054a4f475f3553017904c8a . "./code.idejs/ide-grapher.fasl")
(#x264f3df08635d62bd5f74f37c8b2a15b . "./code.idejs/ide-dev.fasl")
(#x4d28d2cb805f02d8a1d9d55b0107f7bb . "./code.idejs/ide.fasl")
(#x84e302027de9c885371f0d631000e4d6 . "./code.idejs/ide-trace.fasl")
(#x310565e799b494aea04219383cc10414 . "./code.idejs/ide-apropos.fasl")
(#xbd90deac193cc42bd0293a4b2d7c2cc4 . "./code.idejs/ide-inspector.fasl")
(#x4eeead3e74a83f5900cd4686edea21e4 . "./code.idejs/ide-acache.fasl")
(#xab9661d0cdbad4c0579b73f903fcbdc5 . "./code.idejs/ide-project.fasl")
(#x569a4d414344c6b7e634744f5c1cf60c . "./code.idejs/ide-window-list.fasl")
(#x0bed91ae30508e3f9511c53ad6479a30 . "./code.idejs/ide-intro.fasl")
(#xb0278d34b7eb70404a8888b51958fcf3 . "./code.idejs/ide-clipboard.fasl")
(#x58b997c77701a6e91cd91b67674610d2 . "./code.idejs/ide-editor.fasl")
(#x71a0bf2c0b296da87c8d7ec01514a81f . "./code.idejs/ide-process-browser.fasl")
(#x0e36ebbd5d94813d9c22732418c6f168 . "./code.idejs/ide-patch-dialog.fasl")
(#x350a7307bd4c2ab1cbc5a13fee8d5bc0 . "./code.idejs/ide-navigator.fasl")
(#x4a9bad5a7becdca3247e09ac504560a1 . "./cgjs.html")
(#xdac1739373835d1a63cbafa2c3e0b06a . "./ssl-not-found.html")
(#x90c6d2a65504d45266ad9fa3a49be2f9 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js" "code.idejs"))
    (format s "
(let ((path (make-pathname
             :host \"sys\"
             :directory '(:relative \"~a\")
	     :type \"fasl\"
	     )))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows (excl::internal-message-box msg "CG/JS Installation" :type :ok))
